#ifndef __cellBlocks_h_
#define __cellBlocks_h_
#include <map>
#include "adevs.h"
/**
This is a variant of the cellSpace model that partitions the space into
roughly equal-size contiguous blocks.  These blocks are then simulated
as a coupled model.
*/
class cellBlocks: public devsn
{
	public:

		/**
		Create an W x H cellspace with NULL model entries in the cells.
		The number of partitions indicates the number in the x direction
		and y direction (i.e. there will be num_partitions^2 partitions
		in fact).
		*/
		cellBlocks(int num_partitions, int width, int height = 1):
		devsn()
		{
			nx = new net_exec(this,width,height,num_partitions);
		}
		/// Insert a model at the x,y position
		void add(devs* model, int x, int y = 0) { nx->addModel(model,x,y); }
		/// Get the network executive for this coupled model
		netExec* getNetExec() { return nx; }
		/// Get the model at location x,y
		const devs* getModel(int x, int y = 0) const
		{
			return nx->getModel(x,y);
		}
		/// Get the width of the cellspace
		int getWidth() const { return nx->getWidth(); }
		/// Get the height of the cellspace
		int getHeight() const { return nx->getHeight(); }
		/// Destructor
		~cellBlocks(){ delete nx; }
	private:	

		class net_exec: public netExec
		{
			public:
				net_exec(devs* parent, int width, int height, int num_partitions);
				void init();
				void delta_int(){}
				void delta_ext(stime_t e, const adevs_bag<PortValue>& x){}
				void delta_conf(const adevs_bag<PortValue>& x){}
				void output_func(adevs_bag<PortValue>& y){}
				void gc_output(adevs_bag<PortValue>& g){}
				void gc_models(adevs_set<devs*>& g){}
				void getComponents(adevs_set<devs*>& c);
				void addModel(devs* d, int x, int y);
				const devs* getModel(int x, int y) const;
				int getWidth() const { return w; }
				int getHeight() const { return h; }
				void route(const PortValue& pv, devs* model, adevs_bag<EventReceiver>& r);
				~net_exec();
			private:
				int w, h, num_xblocks, num_yblocks;
				cellSpace*** space;
				std::map<cellSpace*,cellSpace::coord_t> coord_table;

				net_exec(const net_exec& src){}
				void net_exec::operator=(const net_exec& src){}
		};
		net_exec* nx;
	
		cellBlocks(){}	
		cellBlocks(const cellBlocks& src){}
		void operator=(const cellBlocks& src){}

};

#endif
