#include<iostream>
#include<cstdio>
#include<cassert>
#include<fstream>
#include<cmath>
#include<vector>
#include "configuration.h"
using namespace std;

static double h = 0.0;
static double tstep = 0.0;
static double Tend = 0.0;
static double k = 0.0;
static int width = 0;
static int height = 0;
static double** u;
static double** u_last;

static void edge_values()
{
	for (int x = 0; x < width; x++)
	{
		u[x][0] = u[x][1];
		u_last[x][0] = u_last[x][1];
		u[x][height-1] = u[x][height-2];
		u_last[x][height-1] = u_last[x][height-2];
	}
	for (int y = 0; y < height; y++)
	{
		u[0][y] = u[1][y];
		u_last[0][y] = u_last[1][y];
		u[width-1][y] = u[width-2][y];
		u_last[width-1][y] = u_last[width-2][y];
	}
}

static void init(string init_file)
{
	configuration config(init_file);
	h = config.get_dx();
	k = config.get_k();
	Tend = config.get_tend();
	width = config.get_width()+2;
	height = config.get_height()+2;
	tstep = config.get_tstep();
	u = new double*[width];
	u_last = new double*[width];
	for (int i = 0; i < width; i++)
	{
		u[i] = new double[height];
		u_last[i] = new double[height];
	}
	for (int x = 1; x < width-1; x++)
	{
		for (int y = 1; y < height-1; y++)
		{
			u[x][y] = u_last[x][y] = config.get_data()[x-1][y-1];
		}
	}
	config.done();
	edge_values();
}
		
static void sample()
{
	ofstream log("soln");
	for(int i = 0; i < width; i++)
	{
		for(int j = 0; j < height; j++)
		{
			log << u[i][j] << " " ;
		}
		log << endl;
	}
	log.close();
}

static void solve()
{
	double t = 0.0;
	for(t = 0.0; t < Tend; t += tstep)
	{
		for(int i = 1; i < width-1; i++)
		{
			for(int j = 1; j < height-1; j++)
			{
				u[i][j] = u_last[i][j]+k*tstep*
				(u_last[i-1][j]+u_last[i+1][j]+u_last[i][j-1]+u_last[i][j+1]-
				4.0*u_last[i][j])/(h*h);
			}
		}
		for(int i = 0; i < width; i++)
		{
			for(int j = 0; j < height; j++)
			{
				u_last[i][j] = u[i][j];
			}
		}
		edge_values();
	}
}

static void clean_up()
{
	for (int i = 0; i < width; i++)
	{
		delete [] u[i];
		delete [] u_last[i];
	}
	delete [] u;
	delete [] u_last;
}

int main(int argc, char* argv[])
{
	if (argc < 2)
	{
		cout << argv[0] << " <config file>" << endl;
		exit(0);
	}
	init(argv[1]);
	solve();
	sample();
	clean_up();
	return 0;
}
