#ifndef __cellspace__h_
#define __cellspace__h_
#include "cell.h"
/**
The cellspace is a network executive containing a grid of cells.  The cell space
wraps at the edges.
*/
class cellspace: public netExec 
{
	public:
		/**
		Constructs a cellspace with the indicated width-2 and height-2.  The
		border cells are used as boundary conditions.
		*/
		cellspace(int width, int height, int** fuel, bool** burning,
		devsn* parent, int xoffset = 0, int yoffset = 0);
		// copy constructor.  Performs a deep copy.
		cellspace(const cellspace& src, devsn* parent);
		// The compute neighhbors method is called by the simulator when messages
		// appear on the output port of a cell.  Returns the cells that form the
		// event generating cell's Moore neighborhood (minus the cell itself).
		void route(const PortValue& pv, devs* src, adevs_bag<EventReceiver>& r);
		// Get all the cells in the space
		void getComponents(adevs_set<devs*>& c);
		void report(ofstream& stream);
		void init() { passivate(); }
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x){}
		void delta_int(){}
		void delta_conf(const adevs_bag<PortValue>& x){}
		void output_func(adevs_bag<PortValue>& y){}
		void gc_output(adevs_bag<PortValue>& g){}
		void gc_models(adevs_set<devs*>& models){}
		~cellspace();

	private:	
		cell** space;
		int xoffset, yoffset;
		int width, height;

		cellspace(){}
		cellspace(const cellspace& src){}
		void operator=(const cellspace& src){}

		friend class world;
};

class world: public devsn
{
	public:
		// Produces output when an edge cell changes state.
		static const port_t left_edge_out;
		static const port_t right_edge_out;
		static const port_t upper_edge_out;
		static const port_t lower_edge_out;
		// Input port used to indicate that a boundary cell has changed state.
		static const port_t left_edge_in;
		static const port_t right_edge_in;
		static const port_t upper_edge_in;
		static const port_t lower_edge_in;

		world(int width, int height, int** fuel, bool** burning,
		int xoffset = 0, int yoffset = 0):
		devsn(),
		nx(width,height,fuel,burning,this,xoffset,yoffset){}
		netExec* getNetExec(){ return &nx; }
		void report(ofstream& log)
		{
			nx.report(log);
		}
		world(const world& src):
		devsn(src),
		nx(src.nx,this)
		{
		}
		object* clone() const { return new world(*this); }
	private:
		cellspace nx;

		world(){}
		void operator=(const world& src){}
};

#endif
