#include <iostream>
#include <fstream>
#include <math.h>
#include <stdlib.h>
#include <string>
#include <algorithm>
using namespace std;

static double* u;
static double* fl;
static double* fr;
static double* s;
static double* wave;
static double tend;
static double dx;
static double dt;
static int LEN;

void Initialize( void )
{
	char str[80];
	/* loading data from file "params.ini" */
	ifstream pF ("params.ini");
	if(pF == 0) 
	{
		cerr << "Cannot open file params.ini" << endl;
		exit( -1 );
   }

   int i = 0;

   do
	{
		pF.getline(str,79);
		if (!pF.eof()) 
		{
			switch(i) 
			{
				case 0: LEN    = atoi( str ); break;
				case 1: dx = atof( str ); break;
				case 2: tend = atof( str ); break;
				case 3: dt = atof( str ); break;
				case 4: dt = atof( str ); break;
				default: break;
			} /* end switch() */
			i++;
		} /* end if() */
		else 
		{
			cerr << "Error while reading file params.ini" << endl;
			exit(1);
		}
	} 
	while( i < 5 );

	u = new double[LEN+2];
	fl = new double[LEN+2];
	fr = new double[LEN+2];
	s = new double[LEN+2];
	wave = new double[LEN+2];

	pF.close();

	/* initial conditions */
	for(i = 0; i < LEN; i++) 
	{
//		u[i] = exp(-1.0*((double)i)*dx);
//		u[i] = 1.0-exp(-1.0*((double)i)*dx);
		double B = 200.0;
		double x = -30.0 + (double)i * dx;
		u[i] = exp(-1/B*(x-0.3)*(x-0.3));
	} 

} /* end Initialize( ) */

void ComputeFluxes()
{
	static const double umax = 1.5;
	for (int i = 1; i < LEN; i++)
	{
		// Compute the wave and speed
		wave[i] = u[i] - u[i-1];
		s[i] = umax * (1.0 - (u[i-1] + u[i]));
		// Compute left-going and right-going flux differences
		if (s[i] < 0.0)
			fl[i] = s[i] * wave[i];
		else
			fl[i] = 0.0;
		if (s[i] > 0.0)
			fr[i] = s[i] * wave[i];
		else
			fr[i] = 0.0;
	}
}

void Integrate()
{
	for (int i = 1; i < LEN; i++)
	{
		u[i] = u[i] - dt/dx*(fl[i+1]+fr[i]);
	}
}

void Report()
{
	ofstream flog("data");
	for (int i = 1; i < LEN-1; i++)
	{
		flog << (double)i*dx << " " << u[i] << " " << fl[i] << " " << endl;	
	}
	flog.close();
}

int main(int argc, char** argv) 
{
	Initialize();
	cout << "tend =  " << tend << endl;
	double t = 0.0;	
	while (t < tend)
	{
		ComputeFluxes();
		Integrate();
		t += dt;
	}
	Report();
	return 0;
}
