#include "multicell.h"
#include <vector>
#include <iostream>
#include <fstream>
#include "perf_timer.h"
#include <math.h>
using namespace std;

static vector<double> data;
static double tend;

void Initialize( void )
{
	int LEN;
	char str[80];
	/* loading data from file "params.ini" */
	ifstream pF ("params.ini");
	if(pF == 0) 
	{
		cerr << "Cannot open file params.ini" << endl;
		exit( -1 );
   }

   int i = 0;

   do
	{
		pF.getline(str,79);
		if (!pF.eof()) 
		{
			switch(i) 
			{
				case 0: LEN    = atoi( str ); break;
				case 1: cell::dx = atof( str ); break;
				case 2: tend = atof( str ); break;
				case 3: cell::D = atof( str ); break;
				default: break;
			} /* end switch() */
			i++;
		} /* end if() */
		else 
		{
			cerr << "Error while reading file params.ini" << endl;
			exit(1);
		}
	} 
	while( i < 4 );

	pF.close();
	double u;

	/* initial conditions */
	for(i = 0; i < LEN; i++) 
	{
//		u = 1.0 - exp(-1.0*((double)i)*cell::dx);
//		u = exp(-1.0*((double)i)*cell::dx);
		double B = 200.0;
		double x = -30.0 + (double)i * cell::dx;
		u = exp(-1/B*(x-0.3)*(x-0.3));
		data.push_back(u);
	} 
	
} /* end Initialize( ) */

int main(int argc, char** argv) 
{
	try 
	{
		Initialize();
		cout << "tend =  " << tend << endl;
		multicell space(data);
		devssim sim(&space);
		tic();
		sim.run(tend);
		double e = tocs();
		cout << e << ", T_min = " << cell::T_min << endl;
		space.report();
	}
	catch(adevs_exception& err) 
	{
		cerr << err.getErrorMsg() << endl;
	}
	return 0;
}
