#include "CDF.h"
#include "params.h"

CDF::CDF():
atomic()
{
}

CDF::CDF(adevs_vec<3> ul, adevs_vec<3> ur, 
bool left_boundary, bool right_boundary):
atomic(),
ul(ul),
ur(ur),
left_boundary(left_boundary),
right_boundary(right_boundary)
{
}

CDF::CDF(const CDF& src):
atomic(src),
ul(src.ul),
ur(src.ur),
f(src.f),
left_boundary(src.left_boundary),
right_boundary(src.right_boundary)
{
}

const CDF& CDF::operator=(const CDF& src)
{
	atomic::operator=(src);
	ul = src.ul;
	ur = src.ur;
	f = src.f;
	left_boundary = src.left_boundary;
	right_boundary = src.right_boundary;
	return *this;
}

void CDF::init()
{
	compute_flux();
	hold(0.0);
}

void CDF::delta_int()
{
	passivate();
}

void CDF::delta_ext(stime_t e, const adevs_bag<PortValue>& x)
{
	delta_conf(x);
}

void CDF::delta_conf(const adevs_bag<PortValue>& x)
{
	for (int i = 0; i < x.getSize(); i++)
	{
		// If input came from the left
		if (x.get(i).port == cell_interface::in[0][1][1])
			ul = x.get(i).value;
		// If input came from the right
		else
			ur = x.get(i).value;
	}
	boundary_conditions();
	compute_flux();
	hold(0.0);
}

void CDF::output_func(adevs_bag<PortValue>& y)
{
	output(out,f,y);
}

void CDF::boundary_conditions()
{
	if (left_boundary)
	{
		ul[0] = ur[0];
		ul[1] = -ur[1];
		ul[2] = ur[2];
	}
	else if (right_boundary)
	{
		ur[0] = ul[0];
		ur[1] = -ul[1];
		ur[2] = ul[2];
	}
}

void CDF::compute_flux() 
{ 
	f = fluxes(ul,ur); 
}
