#include "cell.h"
using namespace std;

#define EPS 1E-10
const port_t cell::in = 0;
const port_t cell::out = 1;
const port_t cell::stats = 2;

void cell::init() 
{
	hold (0.0);
}

void cell::delta_int() 
{
	if (q.empty())
		passivate();
	else 
	{
		while (q.front().first <= EPS) 
		{
			q.pop_front();
			if (q.empty()) 
			{
				passivate();
				return;
			}
		}
		list<pair<double,entity> >::iterator i = q.begin();
		for (; i != q.end(); i++) 
			(*i).first = max(0.0,(*i).first-ta());
		hold (q.front().first);
	}
}

void cell::delta_ext(stime_t e, const adevs_bag<PortValue>& x) 
{
	list<pair<double,entity> >::iterator i = q.begin();
	for (; i != q.end(); i++) 
		(*i).first = max(0.0,(*i).first-elapsed());
	port_iterator in_port(x,in);
	for (; !in_port.end_of_input(); in_port++) 
	{
		entity* e = (entity*)(*in_port);
		q.push_back(pair<double,entity>(e->get_size(),*e));
	}
	q.sort();
	if (timeNext() == INFINITY)
		hold(q.front().first);
}

void cell::delta_conf(const adevs_bag<PortValue>& x) 
{
	if (!q.empty())
	{
		while (q.front().first <= EPS) 
		{
			q.pop_front();
			if (q.empty()) break;
		}
	}
	delta_ext(ta(),x);
}

void cell::output_func(adevs_bag<PortValue>& y) 
{
	list<pair<double,entity> >::const_iterator i = q.begin();
	for (; i != q.end(); i++) 
	{
		if ((*i).first <= EPS)
			output(out,(*i).second.clone(),y);
		else break;
	}
	output(stats,new cell_data(ID,q.size()),y);
}

void cell::gc_output(adevs_bag<PortValue>& g)
{
	for (int i = 0; i < g.getSize(); i++)
		delete g[i].value;
} 
