/**
Solution to du/dt + du/dx = 0 using finite differences
with initial data a triangle
*/
#include <fstream>
#include <iostream>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>

// Discretization in time
static double h = 0.0;

// Discretization in space
static const double k = 1.0;
// Number of cells
#define NUM_CELLS 21
// Time over which to compute the solution
static const double tend = 10.0;
// Cells for computing the solution
static double cells[NUM_CELLS];
// Wave speed at each cell
static double a[NUM_CELLS];

static void initialize(int argc, char** argv)
{
	if (argc < 2)
	{
		cout << argv[0] << " h" << endl;
		exit(-1);
	}
	h = atof(argv[1]);
	if (h <= 0.0)
	{
		cout << "h = " << h << " is too small" << endl;
		exit(-2);
	}
	else
	{
		cout << "h/k = " << h/k << endl;
	}
	int i = 0;
	for (; i < NUM_CELLS; i++)
	{
//		a[i] = 2.0*(double)(NUM_CELLS - i)/(double)NUM_CELLS;
		a[i] = 1.0;
	}
	i = 0; 
/*
	for (; i < NUM_CELLS/4; i++)
	{
		cells[i] = k*(double)i;
	}
	for (; i < NUM_CELLS/2; i++)
	{
		cells[i] = k*(double)(NUM_CELLS/2 - i);
	}
	for (; i < NUM_CELLS; i++)
	{
		cells[i] = 0.0;
	}
*/
	for (; i < NUM_CELLS; i++)
		cells[i] = sin(6.28*(double)i/(double)NUM_CELLS);
}

static void step()
{
	double copy_cells[NUM_CELLS];
	for (int i = 0; i < NUM_CELLS; i++)
	{
		copy_cells[i] = cells[i];
	}
	for (int i = 1; i < NUM_CELLS-1; i++)
	{
		cells[i] = copy_cells[i] - a[i]*h/k*(copy_cells[i]-copy_cells[i-1]);
	}
}

static void record()
{
	static int instance_num = 0;
	char filename[100];
	sprintf(filename,"data-%d",instance_num);
	ofstream f(filename);
	for (int i = 0; i < NUM_CELLS; i++)
		f << (double)i*k << " " << cells[i] << endl;
	f.close();
	instance_num++;
}

static void record_speed()
{
	ofstream f("dtss-speed");
	for (int i = 0; i < NUM_CELLS; i++)
		f << (double)i*k << " " << a[i] << endl;
	f.close();
}

int main(int argc, char** argv)
{
	initialize(argc,argv);
	record_speed();
	int num_steps = (int)ceil(tend/h);
	for (int i = 0; i < num_steps; i++)
	{
		record();
		step();
	}
	return 0;
}
