#include "cellspace.h"

const port_t world::left_edge_out = 1;
const port_t world::right_edge_out = 2;
const port_t world::upper_edge_out = 3;
const port_t world::lower_edge_out = 4;
const port_t world::left_edge_in = 11;
const port_t world::right_edge_in = 12;
const port_t world::upper_edge_in = 13;
const port_t world::lower_edge_in = 14;

cellspace::cellspace(int w, int h, int** fuel, bool** burning,
devsn* parent, int xoffset, int yoffset):
netExec(parent),
xoffset(xoffset),
yoffset(yoffset),
width(w),
height(h)
{
	space = new cell*[width];
	for (int i = 0; i < width; i++)
		space[i] = new cell[height];
	for (int i = 0; i < width; i++)
		for (int j = 0; j < height; j++)
			space[i][j].set_pos(i+xoffset,j+yoffset);
	for (int i = 0; i < w; i++)
	{
		for (int j = 0; j < h; j++)
		{
			space[i][j].set_init_conditions(fuel[i][j],burning[i][j]);
		}
	}
}

cellspace::cellspace(const cellspace& src, devsn* parent):
netExec(src,parent),
xoffset(src.xoffset),
yoffset(src.yoffset),
width(src.width),
height(src.height)
{
	space = new cell*[width];
	for (int i = 0; i < width; i++)
		space[i] = new cell[height];
	for (int i = 0; i < width; i++)
		for (int j = 0; j < height; j++)
			space[i][j] = src.space[i][j];
}	

void cellspace::route(const PortValue& pv, devs* src, adevs_bag<EventReceiver>& r)
{
	int x = pv.value.x-xoffset;
	int y = pv.value.y-yoffset;
	// internal event
	if (src != getParent())
	{
		// send message to neighbors
		for (int i = -1; i < 2; i++)
		{
			for (int j = -1; j < 2; j++)
			{
				if (x+i>=0 && x+i<width && y+j>=0 && y+j<height && (i!=0 || j!=0))
				{
					r.add(EventReceiver(&space[x+i][y+j],cell::in));
				}
			}
		}
		// Edge events
		if (x == 0)
			r.add(EventReceiver(getParent(),world::left_edge_out));
		if (x == width-1)
			r.add(EventReceiver(getParent(),world::right_edge_out));
		if (y == 0)
			r.add(EventReceiver(getParent(),world::upper_edge_out));
		if (y == height-1)
			r.add(EventReceiver(getParent(),world::lower_edge_out));
	}
	// external event
	else
	{
		if (pv.port == world::left_edge_in)
		{
			if (y < height-1) r.add(EventReceiver(&space[0][y+1],cell::in));
			r.add(EventReceiver(&space[0][y],cell::in));
			if (y > 0) r.add(EventReceiver(&space[0][y-1],cell::in));
		}
		else if (pv.port == world::right_edge_in)
		{
			if (y < height-1) r.add(EventReceiver(&space[width-1][y+1],cell::in));
			r.add(EventReceiver(&space[width-1][y],cell::in));
			if (y > 0) r.add(EventReceiver(&space[width-1][y-1],cell::in));
		}
		else if (pv.port == world::upper_edge_in)
		{
			if (x > 0) r.add(EventReceiver(&space[x-1][0],cell::in));
			r.add(EventReceiver(&space[x][0],cell::in));
			if (x < width-1) r.add(EventReceiver(&space[x+1][0],cell::in));
		}
		else if (pv.port == world::lower_edge_in)
		{
			if (x > 0) r.add(EventReceiver(&space[x-1][height-1],cell::in));
			r.add(EventReceiver(&space[x][height-1],cell::in));
			if (x < width-1) r.add(EventReceiver(&space[x+1][height-1],cell::in));
		}
	}
}

void cellspace::getComponents(adevs_set<devs*>& c)
{
	for (int i = 0; i < width; i++)
		for (int j = 0; j < height; j++)
			c.add(&space[i][j]);
}

void cellspace::report(ofstream& stream)
{
	for (int j = 0; j < height; j++)
	{
		for (int i = 0; i < width; i++)
		{
			space[i][j].report(stream);
		}
		stream << endl;
	}
}

cellspace::~cellspace()
{
	for (int i = 0; i < width; i++)
		delete [] space[i];
	delete [] space;
}
