#include <adevs.h>
#include <iostream>
#include <vector>
#ifndef __genr_h_
#define __genr_h_

class genr: public atomic 
{
	public:
		static const port_t start;
		static const port_t stop;
		static const port_t signal;

		genr(const std::vector<stime_t>& pattern, int iterations, bool active = true):
		atomic(),pattern(pattern),active(active),init_state(active),iterations(iterations)
		{}
		genr(stime_t period, int iterations, bool active = true):
		atomic(),active(active),init_state(active),iterations(iterations)
		{
			pattern.push_back(period);
		}
		genr(const genr& src):
		atomic(src),
		pattern(src.pattern),
		active(src.active),
		init_state(src.init_state),
		iterations(src.iterations),
		count(src.count){}
		object* clone() const { return new genr(*this); }
		void init()
		{
			active = init_state;
			count = 1;
			if (!active) passivate();
			else
			{
				assert(pattern[0] >= 0.0);
		 		hold(pattern[0]);
			}
		}
		void delta_int()
		{
			if (count >= iterations) passivate();
			else
			{ 
				assert(pattern[count%pattern.size()] >= 0.0);
				hold(pattern[count++%pattern.size()]);
			}
		}
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x) 
		{
			passivate();
			active = false;
			port_iterator i(x,start);
			if (!i.end_of_input())
			{
				active = true;
				count = 1;
				assert(pattern[0] >= 0.0);
				hold (pattern[0]);
			}
			else
				std::cout << "Got genr.stop" << std::endl;
		}
		void delta_conf (const adevs_bag<PortValue>& x)
		{
			delta_ext(ta(),x);
		}
		void output_func(adevs_bag<PortValue>& y)
		{
			output(signal,new object(),y);
		}
		void gc_output(adevs_bag<PortValue>& g)
		{
			for (int i = 0; i < g.getSize(); i++)
				delete g[i].value;
		}
		~genr(){}

	private:	
		std::vector<stime_t> pattern;
		bool active;
		bool init_state;
		int iterations;
		int count;
};

const port_t genr::start(0);
const port_t genr::stop(1);
const port_t genr::signal(2);

#endif
